<?php

namespace Restaurant\Table\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Restaurant\Table\Http\Requests\TableRequest;
use Restaurant\Table\Interfaces\TableRepositoryInterface;
use Restaurant\Table\Models\Table;

/**
 * Resource controller class for table.
 */
class TableResourceController extends BaseController
{

    /**
     * Initialize table resource controller.
     *
     * @param type TableRepositoryInterface $table
     *
     * @return null
     */
    public function __construct(TableRepositoryInterface $table)
    {
        parent::__construct();
        $this->repository = $table;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Restaurant\Table\Repositories\Criteria\TableResourceCriteria::class);
    }

    /**
     * Display a list of table.
     *
     * @return Response
     */
    public function index(TableRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Restaurant\Table\Repositories\Presenter\TablePresenter::class)
                ->$function();
        }

        $tables = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('table::table.names'))
            ->view('table::table.index', true)
            ->data(compact('tables', 'view'))
            ->output();
    }

    /**
     * Display table.
     *
     * @param Request $request
     * @param Model   $table
     *
     * @return Response
     */
    public function show(TableRequest $request, Table $table)
    {

        if ($table->exists) {
            $view = 'table::table.show';
        } else {
            $view = 'table::table.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('table::table.name'))
            ->data(compact('table'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new table.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(TableRequest $request)
    {

        $table = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('table::table.name')) 
            ->view('table::table.create', true) 
            ->data(compact('table'))
            ->output();
    }

    /**
     * Create new table.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(TableRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $table                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('table::table.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('table/table/' . $table->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/table/table'))
                ->redirect();
        }

    }

    /**
     * Show table for editing.
     *
     * @param Request $request
     * @param Model   $table
     *
     * @return Response
     */
    public function edit(TableRequest $request, Table $table)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('table::table.name'))
            ->view('table::table.edit', true)
            ->data(compact('table'))
            ->output();
    }

    /**
     * Update the table.
     *
     * @param Request $request
     * @param Model   $table
     *
     * @return Response
     */
    public function update(TableRequest $request, Table $table)
    {
        try {
            $attributes = $request->all();

            $table->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('table::table.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('table/table/' . $table->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('table/table/' . $table->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the table.
     *
     * @param Model   $table
     *
     * @return Response
     */
    public function destroy(TableRequest $request, Table $table)
    {
        try {

            $table->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('table::table.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('table/table/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('table/table/' . $table->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple table.
     *
     * @param Model   $table
     *
     * @return Response
     */
    public function delete(TableRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('table::table.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('table/table'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/table/table'))
                ->redirect();
        }

    }

    /**
     * Restore deleted tables.
     *
     * @param Model   $table
     *
     * @return Response
     */
    public function restore(TableRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('table::table.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/table/table'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/table/table/'))
                ->redirect();
        }

    }

}
